/* gcc -Wall md5.c common-opencl.c -o md5 -lOpenCL */
#include <string.h>
#include <stdio.h>
#include "common-opencl.h"

#define PLAINTEXT_LENGTH    31
#define FORMAT_LABEL        "raw-md5-opencl"
#define FORMAT_NAME         "Raw MD5"
#define ALGORITHM_NAME      "OpenCL"
#define BENCHMARK_COMMENT   ""
#define BENCHMARK_LENGTH    -1
#define CIPHERTEXT_LENGTH   32
#define BINARY_SIZE         16
#define SALT_SIZE           0

cl_command_queue queue_prof;
cl_mem pinned_saved_keys, pinned_partial_hashes, buffer_out, buffer_keys,
    data_info;
static cl_uint *partial_hashes;
static char *saved_plain;

#define MIN_KEYS_PER_CRYPT      2048
#define MAX_KEYS_PER_CRYPT      MD5_NUM_KEYS
static unsigned int datai[2];

static void print_hex(unsigned char *str, int len)
{
	int i;
	for (i = 0; i < len; ++i)
		printf("%02x", str[i]);
	printf("\n");
}

static void create_clobj(int kpc)
{
	pinned_saved_keys =
	    clCreateBuffer(context[gpu_id],
	    CL_MEM_READ_WRITE | CL_MEM_ALLOC_HOST_PTR,
	    (PLAINTEXT_LENGTH + 1) * kpc, NULL, &ret_code);
	HANDLE_CLERROR(ret_code,
	    "Error creating page-locked memory pinned_saved_keys");

	saved_plain =
	    (char *) clEnqueueMapBuffer(queue[gpu_id], pinned_saved_keys,
	    CL_TRUE, CL_MAP_WRITE | CL_MAP_READ, 0,
	    (PLAINTEXT_LENGTH + 1) * kpc, 0, NULL, NULL, &ret_code);
	HANDLE_CLERROR(ret_code,
	    "Error mapping page-locked memory saved_plain");

	pinned_partial_hashes = clCreateBuffer(context[gpu_id],
	    CL_MEM_READ_WRITE | CL_MEM_ALLOC_HOST_PTR, 16 * kpc, NULL,
	    &ret_code);
	HANDLE_CLERROR(ret_code,
	    "Error creating page-locked memory pinned_partial_hashes");

	partial_hashes = (cl_uint *) clEnqueueMapBuffer(queue[gpu_id],
	    pinned_partial_hashes, CL_TRUE, CL_MAP_READ, 0, 4 * kpc, 0, NULL,
	    NULL, &ret_code);
	HANDLE_CLERROR(ret_code,
	    "Error mapping page-locked memory partial_hashes");

	// create and set arguments
	buffer_keys = clCreateBuffer(context[gpu_id], CL_MEM_READ_ONLY,
	    (PLAINTEXT_LENGTH + 1) * kpc, NULL, &ret_code);
	HANDLE_CLERROR(ret_code, "Error creating buffer argument buffer_keys");

	buffer_out = clCreateBuffer(context[gpu_id], CL_MEM_WRITE_ONLY,
	    BINARY_SIZE * kpc, NULL, &ret_code);
	HANDLE_CLERROR(ret_code, "Error creating buffer argument buffer_out");

	data_info =
	    clCreateBuffer(context[gpu_id], CL_MEM_READ_ONLY,
	    sizeof(unsigned int) * 2, NULL, &ret_code);
	HANDLE_CLERROR(ret_code, "Error creating data_info out argument");

	HANDLE_CLERROR(clSetKernelArg(crypt_kernel, 0, sizeof(data_info),
		(void *) &data_info), "Error setting argument 0");
	HANDLE_CLERROR(clSetKernelArg(crypt_kernel, 1, sizeof(buffer_keys),
		(void *) &buffer_keys), "Error setting argument 1");
	HANDLE_CLERROR(clSetKernelArg(crypt_kernel, 2, sizeof(buffer_out),
		(void *) &buffer_out), "Error setting argument 2");

	datai[0] = PLAINTEXT_LENGTH;
	datai[1] = kpc;
}

int main()
{
	int kpc = 2048 * 1024;
	unsigned char *p;
	global_work_size = kpc;
	local_work_size = 32;;
	gpu_id = 1;

	opencl_init("md5_kernel.cl", gpu_id, platform_id);
	crypt_kernel = clCreateKernel(program[gpu_id], "md5", &ret_code);
	create_clobj(kpc);

	int i;
	for (i = 0; i < kpc; i++) {
		memcpy(&(saved_plain[i * (PLAINTEXT_LENGTH + 1)]), "openwall",
		    PLAINTEXT_LENGTH + 1);
	}

	// copy keys to the device
	HANDLE_CLERROR(clEnqueueWriteBuffer(queue[gpu_id], data_info, CL_TRUE,
		0, sizeof(unsigned int) * 2, datai, 0, NULL, NULL),
	    "failed in clEnqueueWriteBuffer data_info");
	HANDLE_CLERROR(clEnqueueWriteBuffer(queue[gpu_id], buffer_keys,
		CL_TRUE, 0, (PLAINTEXT_LENGTH + 1) * kpc, saved_plain, 0, NULL,
		NULL), "failed in clEnqueueWriteBuffer buffer_keys");

	HANDLE_CLERROR(clEnqueueNDRangeKernel(queue[gpu_id], crypt_kernel, 1,
		NULL, &global_work_size, &local_work_size, 0, NULL,
		&profilingEvent), "failed in clEnqueueNDRangeKernel");
	HANDLE_CLERROR(clFinish(queue[gpu_id]), "failed in clFinish");

	// read back partial hashes
	HANDLE_CLERROR(clEnqueueReadBuffer(queue[gpu_id], buffer_out, CL_TRUE,
		0, sizeof(cl_uint) * 4 * kpc, partial_hashes, 0, NULL, NULL),
	    "failed in reading data back");

	p = (unsigned char *) partial_hashes;
	print_hex(p, 16);

	return 0;
}
