/*
* This software is Copyright (c) 2012 Lukas Odzioba <ukasz@openwall.net>
* and it is hereby released to the general public under the following terms:
* Redistribution and use in source and binary forms, with or without modification, are permitted.
*
* Code is based on  Aircrack-ng source
*/
#include <string.h>
#include "common-opencl.h"
#include "common.c"

#define uint8_t			unsigned char
#define uint16_t		unsigned short
#define uint32_t		unsigned int

#define PLAINTEXT_LENGTH	15
#define BENCHMARK_COMMENT	""
#define BENCHMARK_LENGTH	-1

typedef struct {
	uint8_t length;
	uint8_t v[15];
} wpapsk_password;

typedef struct {
	uint32_t v[8];
} wpapsk_hash;

typedef struct {
	uint8_t length;
	uint8_t salt[20];
} wpapsk_salt;


static wpapsk_salt currentsalt;		///structure for essid
static wpapsk_password *inbuffer;	///table for candidate passwords
static wpapsk_hash *outbuffer;		///table for PMK calculated by GPU

#define	KEYS_PER_CRYPT		1024 * 2
#define MIN_KEYS_PER_CRYPT	KEYS_PER_CRYPT
#define MAX_KEYS_PER_CRYPT	KEYS_PER_CRYPT
# define SWAP(n) \
    (((n) << 24) | (((n) & 0xff00) << 8) | (((n) >> 8) & 0xff00) | ((n) >> 24))


static cl_mem mem_in, mem_out, mem_setting;
static size_t insize = sizeof(wpapsk_password) * KEYS_PER_CRYPT;
static size_t outsize = sizeof(wpapsk_hash) * KEYS_PER_CRYPT;
static size_t settingsize = sizeof(wpapsk_salt);

static void print_hex(unsigned char *str, int len)
{
	int i;
	for (i = 0; i < len; ++i)
		printf("%02x", str[i]);
	printf("\n");
}


int main()
{
	cl_int cl_error;
	common_init();

	int kpc = 1024 * 2;
	global_work_size = kpc;
	local_work_size = 32;

	gpu_id = 1;

	global_work_size = MAX_KEYS_PER_CRYPT;

	inbuffer = (wpapsk_password *) malloc(sizeof(wpapsk_password) *
			MAX_KEYS_PER_CRYPT);
	outbuffer = (wpapsk_hash *) malloc(sizeof(wpapsk_hash) * MAX_KEYS_PER_CRYPT);

	{
		int i;
		for (i = 0; i < MAX_KEYS_PER_CRYPT; i++)
			inbuffer[i].length = 0;
	}

	opencl_init("wpapsk_kernel.cl", gpu_id, platform_id);
	/// Alocate memory
	mem_in =
	    clCreateBuffer(context[gpu_id], CL_MEM_READ_ONLY, insize, NULL,
	    &cl_error);
	HANDLE_CLERROR(cl_error, "Error alocating mem in");
	mem_setting =
	    clCreateBuffer(context[gpu_id], CL_MEM_READ_ONLY, settingsize,
	    NULL, &cl_error);
	HANDLE_CLERROR(cl_error, "Error alocating mem setting");
	mem_out =
	    clCreateBuffer(context[gpu_id], CL_MEM_WRITE_ONLY, outsize, NULL,
	    &cl_error);
	HANDLE_CLERROR(cl_error, "Error alocating mem out");

	crypt_kernel = clCreateKernel(program[gpu_id], "wpapsk", &cl_error);
	HANDLE_CLERROR(cl_error, "Error creating kernel");
	HANDLE_CLERROR(clSetKernelArg(crypt_kernel, 0, sizeof(mem_in),
		&mem_in), "Error while setting mem_in kernel argument");
	HANDLE_CLERROR(clSetKernelArg(crypt_kernel, 1, sizeof(mem_out),
		&mem_out), "Error while setting mem_out kernel argument");
	HANDLE_CLERROR(clSetKernelArg(crypt_kernel, 2, sizeof(mem_setting),
		&mem_setting), "Error while setting mem_salt kernel argument");

	// set_salt(void *salt)
	char *p = "10f7445c8510fa40d9ef6b4e0f8c772a9d37e449";
	unsigned char s[20] = { 0 };
	int i;
	for (i = 0; i < 20; i++)
		s[i] = atoi16[ARCH_INDEX(p[i * 2])] * 16
			+ atoi16[ARCH_INDEX(p[i * 2 + 1])];


	memcpy(currentsalt.salt, s, 20);
	print_hex(currentsalt.salt, 20);
	currentsalt.length = 20;

/*	for (i = 0; i < kpc; i++) {
		int length = 8;
		memcpy(inbuffer[i * (PLAINTEXT_LENGTH + 1)].v, "openwall", 8);
		inbuffer[i * (PLAINTEXT_LENGTH + 1)].length = 8;
	} */

	memcpy(inbuffer[0].v, "openwall", 8);
	/// Copy data to gpu
	HANDLE_CLERROR(clEnqueueWriteBuffer(queue[gpu_id], mem_in, CL_FALSE, 0,
		insize, inbuffer, 0, NULL, NULL), "Copy data to gpu");
	HANDLE_CLERROR(clEnqueueWriteBuffer(queue[gpu_id], mem_setting,
		CL_FALSE, 0, settingsize, &currentsalt, 0, NULL, NULL),
	    "Copy setting to gpu");

	/// Run kernel
	HANDLE_CLERROR(clEnqueueNDRangeKernel(queue[gpu_id], crypt_kernel, 1,
		NULL, &global_work_size, &local_work_size, 0, NULL, &profilingEvent),
	    "Run kernel");
	HANDLE_CLERROR(clFinish(queue[gpu_id]), "clFinish");

	/// Read the result back
	HANDLE_CLERROR(clEnqueueReadBuffer(queue[gpu_id], mem_out, CL_FALSE, 0,
		outsize, outbuffer, 0, NULL, NULL), "Copy result back");

	/// Await completion of all the above
	HANDLE_CLERROR(clFinish(queue[gpu_id]), "clFinish");

	unsigned char *x;
	x = (unsigned char *) outbuffer;
	print_hex(x, 32);

	return 0;
}
